<?php


namespace IPS\membersshop\modules\admin\points;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * forums
 */
class _forums extends \IPS\Dispatcher\Controller
{
	/**
	 * Execute
	 *
	 * @return	void
	 */
	public function execute()
	{
		
		parent::execute();
	}

	/**
	 * ...
	 *
	 * @return	void
	 */
	protected function manage()
	{
		\IPS\Output::i()->title = \IPS\Member::loggedIn()->language()->addToStack('module__membersshop_points_forums');	   
       
	    $matrix = new \IPS\Helpers\Form\Matrix;
        
        $matrix->manageable = FALSE;
        $matrix->widths = array( 'label' => 30 );
        $matrix->langPrefix = 'ms_forums_';
        
        $matrix->columns = array(
			'label'	=> function( $key, $value, $data )
			{
				return $value;
			}
		);
                
        $matrix->columns[ 'ms_points_topic' ] = function( $key, $value, $data ) 
		{
			return ( is_null( $value ) ) ? NULL : new \IPS\Helpers\Form\Number( 'topics_'.$key, $value, FALSE, array( 'disabled' => (bool) is_null( $value ) ) );
		};                
		$matrix->columns[ 'ms_points_reply' ] = function( $key, $value, $data )
		{
			return ( is_null( $value ) ) ? NULL : new \IPS\Helpers\Form\Number( 'replies_'.$key, $value, FALSE, array( 'disabled' => (bool) is_null( $value ) ) );
		};
		$matrix->columns[ 'ms_points_starter' ] = function( $key, $value, $data )
		{
			return ( is_null( $value ) ) ? NULL : new \IPS\Helpers\Form\Number( 'starter_'.$key, $value, FALSE, array( 'disabled' => (bool) is_null( $value ) ) );
		};

        $rows = array();
		
		foreach( \IPS\forums\Forum::roots( NULL ) AS $root )
		{
			self::populatePermissionMatrix( $rows, $root );
		}
		
		$matrix->rows = $rows;

        if( $values = $matrix->values() )
        {
			if ( !empty( $matrix->changedRows ) )
			{
				foreach ( $matrix->changedRows as $key )
				{
				    $update = array( 'ms_points_topic'   => (int) $values[ $key ]['ms_points_topic'],
                                     'ms_points_reply'   => (int) $values[ $key ]['ms_points_reply'],
                                     'ms_points_starter' => (int) $values[ $key ]['ms_points_starter'] );
                                     
					\IPS\Db::i()->update( 'forums_forums', $update, array( 'id=?', $key ) );
				}
			}
        }
        
		if ( \IPS\Application::appIsEnabled( 'downloads') )
		{
			\IPS\Output::i()->sidebar['actions']['add'] = array(
				'primary'	=> true,
				'icon'		=> 'download',
				'title'		=> 'ms_downloads',
				'link'		=> \IPS\Http\Url::internal( 'app=membersshop&module=points&controller=downloads' ),
				'data'		=> array( )
			);	
		}
		
		if ( \IPS\Settings::i()->clubs )
		{
			\IPS\Output::i()->sidebar['actions']['clubs'] = array(
				'icon'		=> 'users',
				'title'		=> 'ms_clubs',
				'link'		=> \IPS\Http\Url::internal( 'app=membersshop&module=points&controller=clubs' ),
				'data'		=> array( )
			);	
		}
		
		
        \IPS\Output::i()->output = $matrix;
	}
	
	public static function populatePermissionMatrix( &$rows, $forum, $level=0 )
	{
		$rows[ $forum->_id ] = array( '_level' => $level, 'label' => $forum->_title );
        
        if( $forum->sub_can_post AND !$forum->redirect_url )
        {
            $rows[ $forum->_id ]['ms_points_topic']   = (int) $forum->ms_points_topic;
    		$rows[ $forum->_id ]['ms_points_reply']   = (int) $forum->ms_points_reply;
            $rows[ $forum->_id ]['ms_points_starter'] = (int) $forum->ms_points_starter;            
        }
        
        if ( $forum->hasChildren( NULL ) === TRUE )
		{
			$level++;
			foreach( $forum->children( NULL ) AS $child )
			{
				self::populatePermissionMatrix( $rows, $child, $level );
			}
			$level--;
		}
	}
}