<?php


namespace IPS\membersshop\modules\admin\shop;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * custompurchases
 */
class _custompurchases extends \IPS\Dispatcher\Controller
{
	/**
	 * Execute
	 *
	 * @return	void
	 */
	public function execute()
	{
		
		parent::execute();
	}

	/**
	 * ...
	 *
	 * @return	void
	 */
	protected function manage()
	{
		/* Create the table */
		$table = new \IPS\Helpers\Table\Db( 'membersshop_custompurchases', \IPS\Http\Url::internal( 'app=membersshop&module=shop&controller=custompurchases' ) );
		$table->langPrefix = 'ms__';

		/* Column stuff */
		$table->include = array( 'id', 'icon', 'member_id', 'name', 'description', 'price', 'awarded', 'time_date' );
		$table->exclude = array( 'id' );
		$table->mainColumn = 'name';
		$table->widths = array( 'icon' => '5', 'price' => '5', 'time_date' => '5', 'open' => '5', 'awarded' => '5' );
		
		/* Sort stuff */
		$table->sortBy = $table->sortBy ?: 'time_date';
		$table->sortDirection = $table->sortDirection ?: 'asc';
	    $table->noSort	= array( 'icon', 'description' );
        /* Formatters */
		$table->parsers = array(
			'awarded' => function( $val, $row )
			{
				if ( $val == 0)
				{
					$awarded = '<center><i class="fa fa-times fa-lg" aria-hidden="true" style="color:red"></i></center>';
				}
				else if ( $val == 1)
				{
				    $awarded = '<center><i class="fa fa-check fa-lg" aria-hidden="true" style="color:green"></i></center>';
				}
				return $awarded;
			},
			'icon' => function( $val, $row )
			{
				$icon = '<i class="fa '.$val.' fa-lg" aria-hidden="true"></i>';
				return $icon;
			},
			'price' => function( $val, $row )
			{
				$price = number_format($val);
				return $price;
			},
			'time_date' => function( $val, $row )
			{
				$date	= \IPS\DateTime::ts( $val );
				return $date->localeDate() . ' ' . $date->localeTime( FALSE ) ;
			},
			'member_id' => function( $val, $row )
			{
				$member_id = \IPS\Member::load( $val );
				return "<a href='" . \IPS\Http\Url::internal( 'app=core&module=members&controller=members&do=edit&id=' ) . $row['member_id'] . "'>$member_id->name</a>";
				
			}
		);

		$table->rowButtons = function( $row )
		{
			if ( $row['awarded'] == 0 )
			{
				$return = array( 'awarded'	=> array(
					'title'	=> 'Mark Awarded',
					'icon'	=> 'star',
					'link'	=> \IPS\Http\Url::internal( 'app=membersshop&module=shop&controller=custompurchases&do=award&id=' ) . $row['id'],
					'data'	=> array( 'ipsDialog' => '', 'ipsDialog-title' => \IPS\Member::loggedIn()->language()->addToStack('ms_send_pm_popup') )
				)	);
			}
			else
			{
				$return = array();
			}
			return $return;
		};
		
		\IPS\Output::i()->sidebar['actions']['custom'] = array(
		    'primary'   => TRUE,
			'icon'		=> 'cogs',
			'title'		=> 'ms_custompurchases_settings',
			'link'		=> \IPS\Http\Url::internal( 'app=membersshop&module=shop&controller=custompurchases&do=settings' ),
			'data'		=> array( 'ipsDialog' => '', 'ipsDialog-title' => \IPS\Member::loggedIn()->language()->addToStack('ms_custompurchases_settings') )
		);
		
		/* Display */
		\IPS\Output::i()->title = \IPS\Member::loggedIn()->language()->addToStack( 'Custom Purchases' );
		\IPS\Output::i()->output	= (string) $table;
	}
	
	protected function award()
	{
		$form = new \IPS\Helpers\Form();
		$form->add( new \IPS\Helpers\Form\YesNo( 'ms_send_pm', \IPS\Settings::i()->ms_send_pm, FALSE, array('togglesOn' => array( 'ms_pm_content' ) ) ) );
		$form->add( new \IPS\Helpers\Form\Editor( 'ms_pm_content', NULL, FALSE, array( 'app' => 'core', 'key' => 'Admin', 'autoSaveKey' => NULL ), NULL, NULL, NULL, 'ms_pm_content' ) );
		
		if( $values = $form->values() )
		{
			$id  = intval( \IPS\Request::i()->id );
			$items = \IPS\membersshop\Custom::load( $id );
			$receiver = $items->member_id;
				
			if ( $values['ms_send_pm'] )
			{		
				$msgTitle = \IPS\Member::loggedIn()->language()->addToStack( 'ms_pm_title' );
	    	    $msgPost  = $values['ms_pm_content'];
		        \IPS\Member::loggedIn()->language()->parseOutputForDisplay( $msgTitle );
		        \IPS\Member::loggedIn()->language()->parseOutputForDisplay( $msgPost );      

   	     	    try
				{
					$pmSender = \IPS\Member::load( \IPS\Member::loggedIn()->member_id );
				}
				catch( \OutOfRangeException $ex )
				{
	   	         return; 
				} 
        
		        if( !$pmSender->member_id )
		        {
		            return;
		        }            

		        $conversation = \IPS\core\Messenger\Conversation::createItem( $pmSender, $pmSender->ip_address, \IPS\DateTime::ts( time() ) );
		        $conversation->title = $msgTitle;
		        $conversation->is_system = TRUE;
		        $conversation->to_member_id = $receiver;
		        $conversation->save();
            
				$c_members[] = $receiver;
		        $c_members[] = $pmSender->member_id;
			
				$conversation->authorize( $c_members );         
            
				$message = \IPS\core\Messenger\Message::create( $conversation, $msgPost, TRUE, NULL, NULL, $pmSender );
			
				$conversation->first_msg_id = $message->id;
				$conversation->save(); 
			}
			
			$items->awarded = 1;
			$items->save();
			
			\IPS\Output::i()->redirect( \IPS\Http\Url::internal( 'app=membersshop&module=shop&controller=custompurchases' ), 'saved' );
		}
		else
		{
			\IPS\Output::i()->output = $form;
		}
	}
	
	protected function settings()
	{
		$form = new \IPS\Helpers\Form;
		$form->addHeader( 'Notification' );
		$form->add( new \IPS\Helpers\Form\YesNo( 'ms_cp_notifi_members_enable', \IPS\Settings::i()->ms_cp_notifi_members_enable, FALSE, array('togglesOn' => array( 'ms_cp_notifi_members' ) ) ) );
		$form->add( new \IPS\Helpers\Form\Member( 'ms_cp_notifi_members', \IPS\Settings::i()->ms_cp_notifi_members ? array_map( array( 'IPS\Member', 'load' ), explode( ",", \IPS\Settings::i()->ms_cp_notifi_members ) ) : NULL, TRUE, array( 'multiple' => 1 ), NULL, NULL, NULL, 'ms_cp_notifi_members' ) );
		$form->addHeader( 'Email' );
		$form->add( new \IPS\Helpers\Form\YesNo( 'ms_cp_email_members_enable', \IPS\Settings::i()->ms_cp_email_members_enable, FALSE, array('togglesOn' => array( 'ms_cp_email_members' ) ) ) );
		$form->add( new \IPS\Helpers\Form\Member( 'ms_cp_email_members', \IPS\Settings::i()->ms_cp_email_members ? array_map( array( 'IPS\Member', 'load' ), explode( ",", \IPS\Settings::i()->ms_cp_email_members ) ) : NULL, TRUE, array( 'multiple' => 1 ), NULL, NULL, NULL, 'ms_cp_email_members' ) );
		
		if ( $values = $form->values() )
        {
	        $form->saveAsSettings();
	        \IPS\Output::i()->redirect( \IPS\Http\Url::internal( 'app=membersshop&module=shop&controller=custompurchases' ), 'saved' );
	        return TRUE;
        }
		
		\IPS\Output::i()->title = \IPS\Member::loggedIn()->language()->addToStack( 'ms_custompurchases_settings' );
		\IPS\Output::i()->output = $form;
	}
}