<?php


namespace IPS\membersshop\modules\front\shop;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * main
 */
class _main extends \IPS\Dispatcher\Controller
{
	/**
	 * Execute
	 *
	 * @return	void
	 */
	public function execute()
	{
		parent::execute();
		if ( !\IPS\Request::i()->isAjax() AND !\IPS\Settings::i()->IPSDev_Copyright_Remover )
		{
		    $output = "<div style='clear:both;text-align:center;position:absolute;bottom:5px;width:95%;z-index:99999;'><a href='https://www.ipsdev.org/' target='_blank' class='ipsType_light ipsType_smaller'>Members Shop By IPSDev</a></div>";
		    \IPS\Output::i()->output .= $output;  
		}
	}

	/**
	 * ...
	 *
	 * @return	void
	 */
	protected function manage()
	{
		$member = \IPS\Member::loggedIn();
		$cat_id = intval( \IPS\Request::i()->cat_id );
		
		/* Build Table */
		$table = new \IPS\Helpers\Table\Db( 'membersshop_items', \IPS\Request::i()->url() );
		$table->include = array( 'id', 'name', 'price', 'open', 'purchases', 'desc', 'icon', 'item_id', 'position' );
		if ( \IPS\Settings::i()->ms_shop_view_set == 0 )
		{
			$table->rowsTemplate = array( \IPS\Theme::i()->getTemplate('shop'), 'rows' );
		}
		else
		{
			$table->rowsTemplate = array( \IPS\Theme::i()->getTemplate('shop'), 'rowsGrid' );
		}
		$table->joins = array();
		$table->where[] = array( 'open=?', 1 );
		$table->langPrefix = '';
		$table->limit = \IPS\Settings::i()->ms_shop_table_limit;
		if ( \IPS\Settings::i()->ms_sel_what_points == 0 AND $member->member_id )
		{
			$table->title = \IPS\Member::loggedIn()->language()->addToStack('ms_table_header_points', FALSE, array( 'sprintf' => array( number_format( $member->shop_points ) ) ) );
		}
		elseif ( \IPS\Settings::i()->ms_sel_what_points == 1 AND $member->member_id )
		{
			$table->title = 'You have '.number_format( $member->evox_points ).' points';
		}
        $table->sortOptions = array( 'price' => 'price', 'purchases' => 'purchases' );
		$table->sortBy = $table->sortBy ?: 'price';
		if ( $table->sortBy === 'price' )
		{
			$table->sortDirection = 'asc';
		}
		if ( \IPS\Settings::i()->ms_show_sidebar_cat == 1 )
		{
			foreach ( \IPS\Db::i()->select( '*', 'membersshop_item_categories', NULL, 'position ASC' ) as $category )
			{
				$catName = $category['name'];
				$catID = $category['id'];
				$table->filters[ $catName ] = 'membersshop_items.cat_id=' . $catID;
			}
		}
		\IPS\Session::i()->setLocation( \IPS\Http\Url::internal( 'app=membersshop&module=shop&controller=main' ), array(), 'loc_ms_shop' );
		
		try
		{
		    $categories = \IPS\membersshop\Categories::names();
		}
		catch( \UnderflowException $ex )
		{
			$categories = NULL;
		}
		
		\IPS\Output::i()->title = \IPS\Member::loggedIn()->language()->addToStack('ms_browsing_shop');
		\IPS\Output::i()->breadcrumb[] = array( NULL, \IPS\Member::loggedIn()->language()->addToStack('ms_shop') );
		if ( \IPS\Settings::i()->ms_show_sidebar_cat == 0 )
		{
		    \IPS\Output::i()->sidebar['contextual'] = \IPS\Theme::i()->getTemplate( 'shop' )->indexSidebar( $categories );
		}
		\IPS\Output::i()->output = \IPS\Theme::i()->getTemplate('shop')->template( (string) $table, $member );
    }
	
	protected function category()
	{
		$member = \IPS\Member::loggedIn();
		$cat_id = intval( \IPS\Request::i()->id );
		
		/* Build Table */
		$table = new \IPS\Helpers\Table\Db( 'membersshop_items', \IPS\Request::i()->url() );
		$table->include = array( 'id', 'name', 'price', 'open', 'purchases', 'desc', 'icon', 'item_id' );
		if ( \IPS\Settings::i()->ms_shop_view_set == 0 )
		{
			$table->rowsTemplate = array( \IPS\Theme::i()->getTemplate('shop'), 'rows' );
		}
		else
		{
			$table->rowsTemplate = array( \IPS\Theme::i()->getTemplate('shop'), 'rowsGrid' );
		}
		$table->joins = array();
		$table->langPrefix = 'ms__';
		$table->limit = \IPS\Settings::i()->ms_shop_table_limit;
		if ( \IPS\Settings::i()->ms_sel_what_points == 0 AND $member->member_id )
		{
			$table->title = 'You have '.number_format( $member->shop_points ).' points';
		}
		elseif ( \IPS\Settings::i()->ms_sel_what_points == 1 AND $member->member_id )
		{
			$table->title = 'You have '.number_format( $member->evox_points ).' points';
		}
		$table->sortOptions = array( 'price' => 'price', 'purchases' => 'purchases' );
		$table->sortBy = $table->sortBy ?: 'position';
		if ( $table->sortBy === 'position' )
		{
			$table->sortDirection = 'asc';
		}
		
		$table->where[] = array( 'cat_id=? AND open=?', $cat_id, 1 );
		
		/* Location for online list */
		\IPS\Session::i()->setLocation( \IPS\Http\Url::internal( 'app=membersshop&module=shop&controller=main' ), array(), 'loc_ms_shop' );
		
		try
		{
		    $categories = \IPS\membersshop\Categories::names();
		}
		catch( \UnderflowException $ex )
		{
			$categories = NULL;
		}
		
		/* Display */
		\IPS\Output::i()->title = \IPS\Member::loggedIn()->language()->addToStack('ms_browsing_shop');
		\IPS\Output::i()->breadcrumb[] = array( NULL, \IPS\Member::loggedIn()->language()->addToStack('ms_shop') );
		\IPS\Output::i()->sidebar['contextual'] = \IPS\Theme::i()->getTemplate( 'shop' )->indexSidebar( $categories );
		\IPS\Output::i()->output = \IPS\Theme::i()->getTemplate('shop')->template( (string) $table, $member );
    }
	
	protected function buyItem()
	{
		$member = \IPS\Member::loggedIn();
		$id  = intval( \IPS\Request::i()->id );
		
		try
		{
			$perm = \IPS\membersshop\Items::loadAndCheckPerms( $id );
			if( !$perm->can('buy') )
			{
				\IPS\Output::i()->error( 'ms_no_permission_buying', '2F176/1', 404, '' );
			}
		}
		catch ( \OutOfRangeException $e )
		{
			\IPS\Output::i()->error( 'node_error', '2F176/1', 404, '' );
		}
		
		$item = \IPS\membersshop\Items::load( $id );
		
		if ( $item->instock < 1 AND $item->instock != -1 )
		{
			\IPS\Output::i()->error( 'ms_out_of_stock', 'MEMBERSSHOP/1', 403, '' );
		}
		
		if ( !$member->member_id )
		{
			\IPS\Output::i()->error( 'ms_no_module_permission_guest', '2S147/1', 403, '' );
		}
		
		try
		{
			$cost = \IPS\membersshop\Items::load( $id )->price;
		}
		catch( \UnderflowException $ex )
		{
			\IPS\Output::i()->error( 'node_error', '2C163/8', 403, '' );
		}

        if ( \IPS\Settings::i()->ms_sel_what_points == 0 )
		{		
		    if ( $member->shop_points < $cost AND $member->shop_points !== $cost )
		    {
			    \IPS\Output::i()->error( 'ms_you_dont_have_enough_points', '2S147/1', 403, '' );
		    }
			$member->shop_points = $member->shop_points - $cost;
		    $member->save();
		}
        else
        {
            if ( $member->evox_points < $cost AND $member->evox_points !== $cost )
		    {
			    \IPS\Output::i()->error( 'ms_you_dont_have_enough_points', '2S147/1', 403, '' );
		    }
			$member->evox_points = $member->evox_points - $cost;
		    $member->save();
        }
		
		$purchase = new \IPS\membersshop\Purchases;
		$purchase->item_id = $item->item_id;
		$purchase->member_id = $member->member_id;
		$purchase->name = $item->name;
		$purchase->description = $item->desc;
		$purchase->icon = $item->icon;
		$purchase->time_date = time();
		$purchase->price = $cost;
		$purchase->perm_id = $item->id;
		if ( $item->minRand )
		{
			$purchase->minRand = $item->minRand;
		}
		if ( $item->maxRand == 0 or $item->maxRand )
		{
			$purchase->maxRand = $item->maxRand;
		}
		if ( $item->extra_field == 0 or $item->extra_field )
		{
			$purchase->extra_field = $item->extra_field;
		}
		$purchase->save();
		
		$addPurchase = \IPS\membersshop\Items::load( $id );
		$item->purchases = $item->purchases + 1;
		
		if ( $item->instock != -1 )
		{
			$item->instock = $item->instock - 1;
		}
		$item->save();
		
		\IPS\Output::i()->redirect( ( !empty( $_SERVER['HTTP_REFERER'] ) ) ? \IPS\Http\Url::createFromString( $_SERVER['HTTP_REFERER'] ) : \IPS\Http\Url::internal( '' ), \IPS\Member::loggedIn()->language()->addToStack('ms_purchased') );
	}
	
	protected function donatepoints()
	{
		$msmid  = intval( \IPS\Request::i()->msmid );
		$member = \IPS\Member::loggedIn();
		
		if ( $member->member_id == $msmid )
		{
			\IPS\Output::i()->error( 'ms_donate_self', 'MEMBERSHOP/1', 403, '' );
		}
		
		if ( \IPS\Settings::i()->ms_donate_groups == 'all' or \IPS\Member::loggedIn()->inGroup( explode(',', \IPS\Settings::i()->ms_donate_groups ) ) )
		{
			
		}
		else
		{
			\IPS\Output::i()->error( 'ms_no_permission_donate', 'MEMBERSHOP/1', 403, '' );
		}
		
		if ( \IPS\Settings::i()->ms_sel_what_points == 0 )
		{		
			$points = $member->shop_points;
		}
   	    else
		{
		    $points = $member->evox_points;
		}
		
		$form = new \IPS\Helpers\Form();
        $form->addTab('Donate');
		if ( \IPS\Settings::i()->ms_donate_max == -1 )
		{
			$max = NULL;
		}
		else
		{
			$max = \IPS\Settings::i()->ms_donate_max;
		}
		$form = new \IPS\Helpers\Form( 'membersshop', 'Send' );
		$form->add( new \IPS\Helpers\Form\Number('ms_donate_amount', \IPS\Settings::i()->ms_donate_min, FALSE, array( 'min' => \IPS\Settings::i()->ms_donate_min, 'max' => $max ), NULL, NULL, \IPS\Member::loggedIn()->language()->addToStack('ms_points'), 'ms_donate_amount' , function( $val )
   		{
			if ( $points < $val )
			{
		        throw new \DomainException('ms_not_enough_points');
		    }		
   	    }
		));
		
		if ( $values = $form->values() )
		{
			if ( $points < $values['ms_donate_amount'] AND $points != $values['ms_donate_amount']  )
			{
		        \IPS\Output::i()->error( 'ms_you_dont_have_enough_points', '2S147/1', 403, '' );
		    }		
			
			$receiver = \IPS\Member::load( $msmid );
			
			if ( \IPS\Settings::i()->ms_sel_what_points == 0 )
			{		
				$member->shop_points = $member->shop_points - $values['ms_donate_amount'];
				$member->save();
				$receiver->shop_points = $receiver->shop_points + $values['ms_donate_amount'];
				$receiver->save();
			}
   	        else
		    {
		        $member->evox_points = $member->evox_points - $values['ms_donate_amount'];
				$member->save();
				$receiver->evox_points = $receiver->evox_points + $values['ms_donate_amount'];
				$receiver->save();
		    }
			
			$donation = new \IPS\membersshop\Donations;
			$donation->donator_id = $member->member_id;
			$donation->receiver_id = $receiver->member_id;
			$donation->amount = $values['ms_donate_amount'];
			$donation->time_date = time();
			$donation->save();
			
			$items = \IPS\membersshop\Donations::load( $donation->id );
			$notification = new \IPS\Notification( \IPS\Application::load( 'membersshop' ), 'donate_donations', $items, array ( $items ) );
			$notification->recipients->attach( $receiver );
			$notification->send();
			
			\IPS\Output::i()->redirect( isset( $_SERVER['HTTP_REFERER'] ) ? $_SERVER['HTTP_REFERER'] : \IPS\Http\Url::internal( '' ) );
		}
		else
		{
			\IPS\Output::i()->output = \IPS\Theme::i()->getTemplate( 'shop', 'membersshop' )->form( $form );
		}
	}
	
	protected function getDesc()
	{
		$id  = intval( \IPS\Request::i()->id );
		$getItem = \IPS\membersshop\Items::load( $id );
		
		\IPS\Output::i()->output = \IPS\Theme::i()->getTemplate( 'shop', 'membersshop' )->desc( $getItem );
	}
}