<?php


namespace IPS\membersshop\modules\front\shop;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * myitems
 */
class _myitems extends \IPS\Dispatcher\Controller
{
	/**
	 * ...
	 *
	 * @return	void
	 */
	public function execute()
	{
		parent::execute();
		if ( !\IPS\Request::i()->isAjax() AND !\IPS\Settings::i()->IPSDev_Copyright_Remover )
		{
		    $output = "<div style='clear:both;text-align:center;position:absolute;bottom:5px;width:95%;z-index:99999;'><a href='https://www.ipsdev.org/' target='_blank' class='ipsType_light ipsType_smaller'>Members Shop By IPSDev</a></div>";
		    \IPS\Output::i()->output .= $output;  
		}
	}

	/**
	 * ...
	 *
	 * @return	void
	 */
	protected function manage()
	{
		$member = \IPS\Member::loggedIn();

		/* Build Table */
		$table = new \IPS\Helpers\Table\Db( 'membersshop_purchases', \IPS\Http\Url::internal( 'app=membersshop&module=shop&controller=myitems' ) );
		$table->include = array( 'id', 'time_date', 'redeemed', 'price', 'item_id', 'member_id' );
		if ( \IPS\Settings::i()->ms_myitems_view_set == 0 )
		{
			$table->rowsTemplate = array( \IPS\Theme::i()->getTemplate('shop'), 'myitemsrows' );
		}
		else
		{
			$table->rowsTemplate = array( \IPS\Theme::i()->getTemplate('shop'), 'myitemsrowsGrid' );
		}
		$table->joins = array();
		$table->where[] = array( 'redeemed=? AND member_id=?', 0, $member->member_id );
		$table->langPrefix = 'ms__';
		$table->limit = \IPS\Settings::i()->ms_myitems_table_limit;
		$table->sortOptions = array( 'item_id' => 'item_id', 'price' => 'price', 'time_date' => 'time_date' );
		$table->sortBy = $table->sortBy ?: 'item_id';
		if ( $table->sortBy === 'item_id' )
		{
			$table->sortDirection = 'asc';
		}
		$table->title = 'ms_myitems';	
		/* Location for online list */
		\IPS\Session::i()->setLocation( \IPS\Http\Url::internal( 'app=membersshop&module=shop&controller=main' ), array(), 'loc_ms_shop' );
		
		/* Display */
		\IPS\Output::i()->title = \IPS\Member::loggedIn()->language()->addToStack('ms_myitems');
		\IPS\Output::i()->breadcrumb[] = array( NULL, \IPS\Member::loggedIn()->language()->addToStack('ms_myitems') );
		\IPS\Output::i()->output = \IPS\Theme::i()->getTemplate('shop')->template( (string) $table, $member );
	}
	
	protected function redeem()
	{
		$member = \IPS\Member::loggedIn();
		$id  = intval( \IPS\Request::i()->id );
		$timeDate = intval( \IPS\Request::i()->td );
		try
		{
			$getItem = \IPS\membersshop\Purchases::load( $id );
			$item_id = $getItem->item_id;
			if ( $getItem->redeemed == 1 OR $getItem->member_id !== $member->member_id )
			{
				\IPS\Output::i()->error( 'ms_node_error', 'MEMBERSHOP/1', 403, '' );
			}
			
			if ( $item_id == 1 ) { return $this->addPosts(); }
			elseif ( $item_id == 2 ) { return $this->addRep(); }
			elseif ( $item_id == 3 ) { return $this->stealRep(); }
			elseif ( $item_id == 4 ) { return $this->changeUsername(); }
			elseif ( $item_id == 5 ) { return $this->gamblePoints(); }
			elseif ( $item_id == 6 ) { return $this->changeTitle(); }
			elseif ( $item_id == 7 ) { return $this->resetWarnPoints(); }
			elseif ( $item_id == 8 ) { return $this->forumPassword(); }
			elseif ( $item_id == 9 ) { return $this->addSignature(); }
			elseif ( $item_id == 10 ) { return $this->rockpaperscissors(); }
			elseif ( $item_id == 11 ) { \IPS\Output::i()->error( 'ms_11_not_redeemable', 'MEMBERSHOP/1', 403, '' ); }
			elseif ( $item_id == 12 ) { \IPS\Output::i()->error( 'ms_12_not_redeemable', 'MEMBERSHOP/1', 403, '' ); }
			elseif ( $item_id == 13 ) { return $this->upgradeUsergroup(); }
			elseif ( $item_id == 14 ) { return $this->upgradeUsergroup(); }
			elseif ( $item_id == 15 ) { return $this->upgradeUsergroup(); }
			elseif ( $item_id == 16 ) { return $this->gamblePoints(); }
			elseif ( $item_id == 17 ) { return $this->gamblePoints(); }
			elseif ( $item_id == 18 ) { return $this->randomTrophy(); }
			elseif ( $item_id == 19 ) { return $this->uploadAvatar(); }
			elseif ( $item_id == 20 ) { return $this->goAnonymous(); }
			elseif ( $item_id == 21 ) { return $this->guesstheNumber(); }
			elseif ( $item_id > 99 ) { return $this->customPurchase(); }
		}
		catch( \UnderflowException $ex )
		{
			\IPS\Output::i()->error( 'node_error', '2C163/8', 403, '' );
		}
	}
	
	protected function sendTo()
	{
		if ( \IPS\Settings::i()->ms_group_send_items == 'all' or \IPS\Member::loggedIn()->inGroup( explode(',', \IPS\Settings::i()->ms_group_send_items ) ) )
		{
			
		}
		else
		{
			\IPS\Output::i()->error( 'ms_no_permission_send_items', 'MEMBERSHOP/1', 403, '' );
		}
		
		$member = \IPS\Member::loggedIn();
		$id  = intval( \IPS\Request::i()->id );
		$item_id  = intval( \IPS\Request::i()->item_id );
		$permission = intval( \IPS\Request::i()->perm );
		
		try
		{
			$perm = \IPS\membersshop\Items::loadAndCheckPerms( $permission );
			if( !$perm->can('send') )
			{
				\IPS\Output::i()->error( 'ms_no_permission_sending', '2F176/1', 404, '' );
			}
		}
		catch ( \OutOfRangeException $e )
		{
			\IPS\Output::i()->error( 'node_error', '2F176/1', 404, '' );
		}
		
		
		try
		{
			$getItem = \IPS\membersshop\Purchases::load( $id );
			if ( $getItem->redeemed == 1 OR $getItem->member_id !== $member->member_id )
			{
				\IPS\Output::i()->error( 'ms_node_error', 'MEMBERSHOP/1', 403, '' );
			}
		}
		catch( \UnderflowException $ex )
		{
			\IPS\Output::i()->error( 'node_error', '2C163/8', 403, '' );
		}
		
		try
		{
			$percent = \IPS\Settings::i()->ms_charge_to_send;
			$cost = $getItem->price / $percent;
		}
		catch( \UnderflowException $ex )
		{
			\IPS\Output::i()->error( 'node_error', '2C163/8', 403, '' );
		}
		
		$form = new \IPS\Helpers\Form( 'membersshop', 'Send' );
		$form->add(new \IPS\Helpers\Form\Member('ms_send_to', NULL, TRUE, array( ), function( $val )
        {
			if ( $val == \IPS\Member::loggedIn() )
	        {
		        throw new \DomainException('ms_SendSelf');
	        }
        }
		));
				
		if ( $values = $form->values() )
		{
			
			if ( \IPS\Settings::i()->ms_sel_what_points == 0 )
			{		
			    if ( $member->shop_points < $cost AND $member->shop_points !== $cost )
			    {
				    \IPS\Output::i()->error( 'ms_you_dont_have_enough_points', '2S147/1', 403, '' );
			    }
				$member->shop_points = $member->shop_points - $cost;
			    $member->save();
			}
	        else
	        {
	            if ( $member->evox_points < $cost AND $member->evox_points !== $cost )
			    {
				    \IPS\Output::i()->error( 'ms_you_dont_have_enough_points', '2S147/1', 403, '' );
			    }
				$member->evox_points = $member->evox_points - $cost;
			    $member->save();
	        }
		
			$getItem->member_id = $values['ms_send_to']->member_id;
			$getItem->save();
			
			$received = new \IPS\membersshop\Received;
			$received->item_id = $item_id;
			$received->sender_id = $member->member_id;
			$received->receiver_id = $values['ms_send_to']->member_id;
			$received->time_date = time();
			$received->save();
			
			$items = \IPS\membersshop\Received::load( $received->id );
			
			$memberssssss = \IPS\Member::load( $values['ms_send_to']->member_id );
			$notification = new \IPS\Notification( \IPS\Application::load( 'membersshop' ), 'received_item', $items, array ( $items ) );
			$notification->recipients->attach( $memberssssss );
			$notification->send();
			
			\IPS\Output::i()->redirect( ( !empty( $_SERVER['HTTP_REFERER'] ) ) ? \IPS\Http\Url::createFromString( $_SERVER['HTTP_REFERER'] ) : \IPS\Http\Url::internal( '' ), 'Item Sent' );
		}
		else
		{
			\IPS\Output::i()->output = $form->customTemplate( array( call_user_func_array( array( \IPS\Theme::i(), 'getTemplate' ), array( 'forms', 'core' ) ), 'popupTemplate' ) );
		}
	}
	
	protected function addPosts()
	{
		$member = \IPS\Member::loggedIn();
		$id  = intval( \IPS\Request::i()->id );
		$item_id  = intval( \IPS\Request::i()->item_id );
		$timeDate = intval( \IPS\Request::i()->td );
		try
		{
			$getItem = \IPS\membersshop\Purchases::load( $id );
			if ( $getItem->redeemed == 1 OR $getItem->member_id !== $member->member_id )
			{
				\IPS\Output::i()->error( 'ms_node_error', 'MEMBERSHOP/1', 403, '' );
			}
		}
		catch( \UnderflowException $ex )
		{
			\IPS\Output::i()->error( 'node_error', '2C163/8', 403, '' );
		}
		
		$minRand = $getItem->minRand;
		$maxRand = $getItem->maxRand;
		
		$rand = rand($minRand,$maxRand);
		$member->member_posts = $member->member_posts + $rand;
		$member->save();
		
		$getItem->redeemed = 1;
		$getItem->save();
		
		$reward = new \IPS\membersshop\Rewards;
		$reward->item_id = $item_id;
		$reward->member_id = $member->member_id;
		$reward->time_date = time();
		$reward->reward = $rand;
		$reward->name = $getItem->name;
		$reward->description = $getItem->description;
		$reward->save();
		
		$outputLang = ( number_format( $rand ) ) ? \IPS\Member::loggedIn()->language()->addToStack('ms_posts_awarded', FALSE, array( 'sprintf' => array( number_format( $rand ) ) ) ) : 'saved';
   	    \IPS\Output::i()->redirect( ( !empty( $_SERVER['HTTP_REFERER'] ) ) ? \IPS\Http\Url::createFromString( $_SERVER['HTTP_REFERER'] ) : \IPS\Http\Url::internal( '' ), $outputLang );
	}
	
	protected function addRep()
	{
		$member = \IPS\Member::loggedIn();
		$id  = intval( \IPS\Request::i()->id );
		$item_id  = intval( \IPS\Request::i()->item_id );
		$timeDate = intval( \IPS\Request::i()->td );
		try
		{
			$getItem = \IPS\membersshop\Purchases::load( $id );
			if ( $getItem->redeemed == 1 OR $getItem->member_id !== $member->member_id )
			{
				\IPS\Output::i()->error( 'ms_node_error', 'MEMBERSHOP/1', 403, '' );
			}
		}
		catch( \UnderflowException $ex )
		{
			\IPS\Output::i()->error( 'node_error', '2C163/8', 403, '' );
		}
		
		$minRand = $getItem->minRand;
		$maxRand = $getItem->maxRand;
		
		$rand = rand($minRand,$maxRand);
		$member->pp_reputation_points = $member->pp_reputation_points + $rand;
		$member->save();
		
		$getItem->redeemed = 1;
		$getItem->save();
		
		$reward = new \IPS\membersshop\Rewards;
		$reward->item_id = $item_id;
		$reward->member_id = $member->member_id;
		$reward->time_date = time();
		$reward->reward = $rand;
		$reward->name = $getItem->name;
		$reward->description = $getItem->description;
		$reward->save();
		
		$outputLang = ( number_format( $rand ) ) ? \IPS\Member::loggedIn()->language()->addToStack('ms_rep_awarded', FALSE, array( 'sprintf' => array( number_format( $rand ) ) ) ) : 'saved';
   	    \IPS\Output::i()->redirect( ( !empty( $_SERVER['HTTP_REFERER'] ) ) ? \IPS\Http\Url::createFromString( $_SERVER['HTTP_REFERER'] ) : \IPS\Http\Url::internal( '' ), $outputLang );
	}
	
	protected function stealRep()
	{
		$member = \IPS\Member::loggedIn();
		$id  = intval( \IPS\Request::i()->id );
		$item_id  = intval( \IPS\Request::i()->item_id );
		$timeDate = intval( \IPS\Request::i()->td );
		try
		{
			$getItem = \IPS\membersshop\Purchases::load( $id );
			if ( $getItem->redeemed == 1 OR $getItem->member_id !== $member->member_id )
			{
				\IPS\Output::i()->error( 'ms_node_error', 'MEMBERSHOP/1', 403, '' );
			}
		}
		catch( \UnderflowException $ex )
		{
			\IPS\Output::i()->error( 'node_error', '2C163/8', 403, '' );
		}
		
		$form = new \IPS\Helpers\Form( 'membersshop', 'Steal' );
		$form->add(new \IPS\Helpers\Form\Member('ms_stealfrom', NULL, TRUE, array( ), function( $val )
        {
			if ( $val == \IPS\Member::loggedIn() )
	        {
		        throw new \DomainException('ms_StealSelf');
	        }
        }
		));
		
		if ( $values = $form->values( ) )
		{
			$minRand = $getItem->minRand;
		    $maxRand = $getItem->maxRand;
		
		    $stealingfrom = $values['ms_stealfrom'];
		    $rand = rand($minRand,$maxRand);
		    $member->pp_reputation_points = $member->pp_reputation_points + $rand;
		    $member->save();
			$stealingfrom->pp_reputation_points = $stealingfrom->pp_reputation_points - $rand;
			$stealingfrom->save();
			
			$getItem->redeemed = 1;
		    $getItem->save();
		
		    $reward = new \IPS\membersshop\Rewards;
		    $reward->item_id = $item_id;
		    $reward->member_id = $member->member_id;
		    $reward->time_date = time();
		    $reward->reward = $rand ;
			$reward->stolen_from = $stealingfrom->member_id;
			$reward->name = $getItem->name;
		    $reward->description = $getItem->description;
		    $reward->save();
		
		    $noti = \IPS\Db::i()->select( 'id', 'membersshop_rewards', array( 'item_id=? AND member_id=? AND time_date=? AND reward=?', $item_id, $member->member_id, time(), $rand ) )->first();
		    $items = \IPS\membersshop\Rewards::load( $noti );
			
			$memberssssss = \IPS\Member::load( $values['ms_stealfrom']->member_id );
			$notification = new \IPS\Notification( \IPS\Application::load( 'membersshop' ), 'stolen_rep', $items, array ( $items ) );
			$notification->recipients->attach( $memberssssss );
			$notification->send();	
			
			$outputLang = ( number_format( $rand ) ) ? \IPS\Member::loggedIn()->language()->addToStack('ms_rep_stolen', FALSE, array( 'sprintf' => array( number_format( $rand ) ) ) ) : 'saved';
   	        \IPS\Output::i()->redirect( ( !empty( $_SERVER['HTTP_REFERER'] ) ) ? \IPS\Http\Url::createFromString( $_SERVER['HTTP_REFERER'] ) : \IPS\Http\Url::internal( '' ), $outputLang );
		}
		else
		{
			\IPS\Output::i()->output = $form->customTemplate( array( call_user_func_array( array( \IPS\Theme::i(), 'getTemplate' ), array( 'forms', 'core' ) ), 'popupTemplate' ) );
		}
	}
	
	protected function changeUsername()
	{
		$member = \IPS\Member::loggedIn();
		$id  = intval( \IPS\Request::i()->id );
		$item_id  = intval( \IPS\Request::i()->item_id );
		$timeDate = intval( \IPS\Request::i()->td );
		try
		{
			$getItem = \IPS\membersshop\Purchases::load( $id );
			if ( $getItem->redeemed == 1 OR $getItem->member_id !== $member->member_id )
			{
				\IPS\Output::i()->error( 'ms_node_error', 'MEMBERSHOP/1', 403, '' );
			}
		}
		catch( \UnderflowException $ex )
		{
			\IPS\Output::i()->error( 'node_error', '2C163/8', 403, '' );
		}
		
		$form = new \IPS\Helpers\Form( 'form', 'Change', NULL, array( 'data-controller' => 'core.front.system.register') );
		$form->add( new \IPS\Helpers\Form\Text( 'username', NULL, TRUE, array( 'accountUsername' => TRUE ) ) );
		
		if ( $values = $form->values( ) )
		{
		    $member->name = $values['username'];
			$member->members_seo_name = \IPS\Http\Url\Friendly::seoTitle( $values['username'] );
		    $member->save();
		    
			$getItem->redeemed = 1;
		    $getItem->save();

            $reward = new \IPS\membersshop\Rewards;
			$reward->item_id = $item_id;
			$reward->member_id = $member->member_id;
			$reward->time_date = time();
			$reward->name = $getItem->name;
		    $reward->description = $getItem->description;
		    $reward->save();

            $outputLang = ( $member->name ) ? \IPS\Member::loggedIn()->language()->addToStack('ms_changed_username', FALSE, array( 'sprintf' => array( $member->name ) ) ) : 'saved';
   	        \IPS\Output::i()->redirect( ( !empty( $_SERVER['HTTP_REFERER'] ) ) ? \IPS\Http\Url::createFromString( $_SERVER['HTTP_REFERER'] ) : \IPS\Http\Url::internal( '' ), $outputLang );
		}
		else
		{
			\IPS\Output::i()->output = $form->customTemplate( array( call_user_func_array( array( \IPS\Theme::i(), 'getTemplate' ), array( 'forms', 'core' ) ), 'popupTemplate' ) );
		}
	}
	
	protected function gamblePoints()
	{
		$member = \IPS\Member::loggedIn();
		$id  = intval( \IPS\Request::i()->id );
		$item_id  = intval( \IPS\Request::i()->item_id );
		$timeDate = intval( \IPS\Request::i()->td );
		try
		{
			$getItem = \IPS\membersshop\Purchases::load( $id );
			if ( $getItem->redeemed == 1 OR $getItem->member_id !== $member->member_id )
			{
				\IPS\Output::i()->error( 'ms_node_error', 'MEMBERSHOP/1', 403, '' );
			}
		}
		catch( \UnderflowException $ex )
		{
			\IPS\Output::i()->error( 'node_error', '2C163/8', 403, '' );
		}
		
		$minRand = $getItem->minRand;
		$maxRand = $getItem->maxRand;
			
		$rand = rand($minRand,$maxRand);
		if ( \IPS\Settings::i()->ms_sel_what_points == 0 )
		{
		    $member->shop_points = $member->shop_points + $rand;
		}
		else
		{
			$member->evox_points = $member->evox_points + $rand;
		}
		$member->save();
		
		$getItem->redeemed = 1;
		$getItem->save();
		
		$reward = new \IPS\membersshop\Rewards;
		$reward->item_id = $item_id;
		$reward->member_id = $member->member_id;
		$reward->time_date = time();
		$reward->reward = $rand ;
		$reward->name = $getItem->name;
		$reward->description = $getItem->description;
		$reward->save();
		
		$outputLang = ( number_format( $rand ) ) ? \IPS\Member::loggedIn()->language()->addToStack('ms_points_won', FALSE, array( 'sprintf' => array( number_format( $rand ) ) ) ) : 'saved';
   	    \IPS\Output::i()->redirect( ( !empty( $_SERVER['HTTP_REFERER'] ) ) ? \IPS\Http\Url::createFromString( $_SERVER['HTTP_REFERER'] ) : \IPS\Http\Url::internal( '' ), $outputLang );
	}
	
	protected function changeTitle()
	{
		$member = \IPS\Member::loggedIn();
		$id  = intval( \IPS\Request::i()->id );
		$item_id  = intval( \IPS\Request::i()->item_id );
		$timeDate = intval( \IPS\Request::i()->td );
		try
		{
			$getItem = \IPS\membersshop\Purchases::load( $id );
			if ( $getItem->redeemed == 1 OR $getItem->member_id !== $member->member_id )
			{
				\IPS\Output::i()->error( 'ms_node_error', 'MEMBERSHOP/1', 403, '' );
			}
		}
		catch( \UnderflowException $ex )
		{
			\IPS\Output::i()->error( 'node_error', '2C163/8', 403, '' );
		}
		
		$form = new \IPS\Helpers\Form( 'form', 'Change', NULL, array( 'data-controller' => 'core.front.system.register') );
		$form->add( new \IPS\Helpers\Form\Text( 'ms_title', $member->member_title, TRUE, array( ) ) );
		
		if ( $values = $form->values( ) )
		{
		    $member->member_title = $values['ms_title'];
			$member->save();
		    
			$getItem->redeemed = 1;
		    $getItem->save();
			
			$reward = new \IPS\membersshop\Rewards;
			$reward->item_id = $item_id;
			$reward->member_id = $member->member_id;
			$reward->time_date = time();
			$reward->name = $getItem->name;
		    $reward->description = $getItem->description;
			$reward->save();
			
			$outputLang = ( $member->member_title ) ? \IPS\Member::loggedIn()->language()->addToStack('ms_changed_title', FALSE, array( 'sprintf' => array( $member->member_title ) ) ) : 'saved';
   	        \IPS\Output::i()->redirect( ( !empty( $_SERVER['HTTP_REFERER'] ) ) ? \IPS\Http\Url::createFromString( $_SERVER['HTTP_REFERER'] ) : \IPS\Http\Url::internal( '' ), $outputLang );
		}
		else
		{
			\IPS\Output::i()->output = $form->customTemplate( array( call_user_func_array( array( \IPS\Theme::i(), 'getTemplate' ), array( 'forms', 'core' ) ), 'popupTemplate' ) );
		}
	}
	
	protected function resetWarnPoints()
	{
		$member = \IPS\Member::loggedIn();
		$id  = intval( \IPS\Request::i()->id );
		$item_id  = intval( \IPS\Request::i()->item_id );
		$timeDate = intval( \IPS\Request::i()->td );
		try
		{
			$getItem = \IPS\membersshop\Purchases::load( $id );
			if ( $getItem->redeemed == 1 OR $getItem->member_id !== $member->member_id )
			{
				\IPS\Output::i()->error( 'ms_node_error', 'MEMBERSHOP/1', 403, '' );
			}
		}
		catch( \UnderflowException $ex )
		{
			\IPS\Output::i()->error( 'node_error', '2C163/8', 403, '' );
		}
		
		$member->warn_level = 0;
		$member->save();
		    
		$getItem->redeemed = 1;
		$getItem->save();
		
		$reward = new \IPS\membersshop\Rewards;
		$reward->item_id = $item_id;
		$reward->member_id = $member->member_id;
		$reward->time_date = time();
		$reward->name = $getItem->name;
		$reward->description = $getItem->description;
		$reward->save();
		
		\IPS\Output::i()->redirect( ( !empty( $_SERVER['HTTP_REFERER'] ) ) ? \IPS\Http\Url::createFromString( $_SERVER['HTTP_REFERER'] ) : \IPS\Http\Url::internal( '' ), 'ms_reset_warning_points' );
	}
	
	protected function forumPassword()
	{
		$member = \IPS\Member::loggedIn();
		$id  = intval( \IPS\Request::i()->id );
		$item_id  = intval( \IPS\Request::i()->item_id );
		$timeDate = intval( \IPS\Request::i()->td );
		try
		{
			$getItem = \IPS\membersshop\Purchases::load( $id );
			if ( $getItem->redeemed == 1 OR $getItem->member_id !== $member->member_id )
			{
				\IPS\Output::i()->error( 'ms_node_error', 'MEMBERSHOP/1', 403, '' );
			}
		}
		catch( \UnderflowException $ex )
		{
			\IPS\Output::i()->error( 'node_error', '2C163/8', 403, '' );
		}
		
		$form = new \IPS\Helpers\Form( 'membersshop', 'Show Password' );
		$form->add( new \IPS\Helpers\Form\Node( 'ms_ForumCat', NULL, FALSE, array( 'class' => 'IPS\forums\Forum', 'permissionCheck' => function ( $forum ) { return $forum->sub_can_post and !$forum->redirect_url and $forum->password; } ), NULL, NULL, NULL, 'ms_ForumCat' ) );
		
		if( $values = $form->values() )
		{
		    $forum_id      = $values['ms_ForumCat']->id;
			$getItem->redeemed = 1;
		    $getItem->save();
			
			$reward = new \IPS\membersshop\Rewards;
			$reward->item_id = $item_id;
			$reward->member_id = $member->member_id;
			$reward->time_date = time();
			$reward->name = $getItem->name;
		    $reward->description = $getItem->description;
			$reward->save();
			
            \IPS\Output::i()->output = \IPS\Theme::i()->getTemplate( 'shop', 'membersshop' )->forumpassword( $forum_id );
		}
		else
		{
			\IPS\Output::i()->output = $form->customTemplate( array( call_user_func_array( array( \IPS\Theme::i(), 'getTemplate' ), array( 'forms', 'core' ) ), 'popupTemplate' ) );
		}
	}
	
	protected function addSignature()
	{
		$member = \IPS\Member::loggedIn();
		$id  = intval( \IPS\Request::i()->id );
		$item_id  = intval( \IPS\Request::i()->item_id );
		$timeDate = intval( \IPS\Request::i()->td );
		try
		{
			$getItem = \IPS\membersshop\Purchases::load( $id );
			if ( $getItem->redeemed == 1 OR $getItem->member_id !== $member->member_id )
			{
				\IPS\Output::i()->error( 'ms_node_error', 'MEMBERSHOP/1', 403, '' );
			}
		}
		catch( \UnderflowException $ex )
		{
			\IPS\Output::i()->error( 'node_error', '2C163/8', 403, '' );
		}
		
		$form = new \IPS\Helpers\Form( 'membersshop', 'Save' );
		$form->add( new \IPS\Helpers\Form\Editor( 'signature',  $member->signature, FALSE, array( 'app' => 'core', 'key' => 'Signatures', 'autoSaveKey' => "frontsig-" . $member->member_id, 'attachIds' => array(  $member->member_id ) ) ) );
		
		if( $values = $form->values() )
		{
		    \IPS\Db::i()->update( 'core_members', array( 'signature' => $values['signature'] ), array ( 'member_id=?', $member->member_id ) );
			$getItem->redeemed = 1;
		    $getItem->save();
			
			$reward = new \IPS\membersshop\Rewards;
			$reward->item_id = $item_id;
			$reward->member_id = $member->member_id;
			$reward->time_date = time();
			$reward->name = $getItem->name;
		    $reward->description = $getItem->description;
			$reward->save();
			
			\IPS\Output::i()->redirect( ( !empty( $_SERVER['HTTP_REFERER'] ) ) ? \IPS\Http\Url::createFromString( $_SERVER['HTTP_REFERER'] ) : \IPS\Http\Url::internal( '' ), 'ms_changed_signature' );
		}
		else
		{
			\IPS\Output::i()->output = $form->customTemplate( array( call_user_func_array( array( \IPS\Theme::i(), 'getTemplate' ), array( 'forms', 'core' ) ), 'popupTemplate' ) );
		}
	}
	
	
	protected function rockpaperscissors()
	{
		$member = \IPS\Member::loggedIn();
		$id  = intval( \IPS\Request::i()->id );
		$item_id  = intval( \IPS\Request::i()->item_id );
		$timeDate = intval( \IPS\Request::i()->td );
		try
		{
			$getItem = \IPS\membersshop\Purchases::load( $id );
			if ( $getItem->redeemed == 1 OR $getItem->member_id !== $member->member_id )
			{
				\IPS\Output::i()->error( 'ms_node_error', 'MEMBERSHOP/1', 403, '' );
			}
		}
		catch( \UnderflowException $ex )
		{
			\IPS\Output::i()->error( 'node_error', '2C163/8', 403, '' );
		}
		
		$form = new \IPS\Helpers\Form( 'membersshop', \IPS\Member::loggedIn()->language()->addToStack('ms_play') );
		$form->addMessage('ms_rpsls_desc');
		$form->add( new \IPS\Helpers\Form\Radio( 'ms_rps', NULL, FALSE, array( 'options' => array( 1 => \IPS\Member::loggedIn()->language()->addToStack('ms_rock'), 2 => \IPS\Member::loggedIn()->language()->addToStack('ms_paper'), 3 => \IPS\Member::loggedIn()->language()->addToStack('ms_scissors'), 4 => \IPS\Member::loggedIn()->language()->addToStack('ms_lizard'), 5 => \IPS\Member::loggedIn()->language()->addToStack('ms_spock') )  ) ) );
		
		if ( $getItem->extra_field )
		{
		    $form->addMessage('ms_rpsls_how_much_to_win');
			$form->add( new \IPS\Helpers\Form\Number('ms_rpsls_amount', NULL, FALSE, array( 'max' => $getItem->maxRand ), NULL, NULL, \IPS\Member::loggedIn()->language()->addToStack('ms_points'), 'ms_rpsls_amount' , function( $val )
   		    {
				if ( \IPS\Settings::i()->ms_sel_what_points == 0 )
				{
					if ( \IPS\Member::loggedIn()->shop_points < $val )
					{
				        throw new \DomainException('ms_not_enough_points');
		    	    }
				}
				else
				{
					if ( \IPS\Member::loggedIn()->evox_points < $val )
					{
				        throw new \DomainException('ms_not_enough_points');
		    	    }
				}
   	        }
			));
	    }
		
		if( $values = $form->values() )
		{
			if ( $getItem->extra_field )
		    {
			    if ( $values['ms_rpsls_amount'] > 0 )
				{
					if ( \IPS\Settings::i()->ms_sel_what_points == 0 )
					{
					    $member->shop_points = $member->shop_points - $values['ms_rpsls_amount'];
					}
					else
					{
						$member->evox_points = $member->evox_points - $values['ms_rpsls_amount'];
					}
					$member->save();
			
					$getItem->price = $getItem->price + $values['ms_rpsls_amount'];
					$getItem->save();
			    }
			}
			
		    $rand = rand(1,5);
			
			    if ( $rand == 1 AND $values['ms_rps'] == 1) { $weThrew = 1; $youthrew = 1; $outcome = 333; }
			elseif ( $rand == 1 AND $values['ms_rps'] == 2) { $weThrew = 1; $youthrew = 2; $outcome = 111; }
			elseif ( $rand == 1 AND $values['ms_rps'] == 3) { $weThrew = 1; $youthrew = 3; $outcome = 222; }
			elseif ( $rand == 1 AND $values['ms_rps'] == 4) { $weThrew = 1; $youthrew = 4; $outcome = 222; }
			elseif ( $rand == 1 AND $values['ms_rps'] == 5) { $weThrew = 1; $youthrew = 5; $outcome = 111; }
			
			elseif ( $rand == 2 AND $values['ms_rps'] == 1) { $weThrew = 2; $youthrew = 1; $outcome = 222; }
			elseif ( $rand == 2 AND $values['ms_rps'] == 2) { $weThrew = 2; $youthrew = 2; $outcome = 333; }
			elseif ( $rand == 2 AND $values['ms_rps'] == 3) { $weThrew = 2; $youthrew = 3; $outcome = 111; }
			elseif ( $rand == 2 AND $values['ms_rps'] == 4) { $weThrew = 2; $youthrew = 4; $outcome = 111; }
			elseif ( $rand == 2 AND $values['ms_rps'] == 5) { $weThrew = 2; $youthrew = 5; $outcome = 222; }
			
			elseif ( $rand == 3 AND $values['ms_rps'] == 1) { $weThrew = 3; $youthrew = 1; $outcome = 111; }
			elseif ( $rand == 3 AND $values['ms_rps'] == 2) { $weThrew = 3; $youthrew = 2; $outcome = 222; }
			elseif ( $rand == 3 AND $values['ms_rps'] == 3) { $weThrew = 3; $youthrew = 3; $outcome = 333; }
			elseif ( $rand == 3 AND $values['ms_rps'] == 4) { $weThrew = 3; $youthrew = 4; $outcome = 222; }
			elseif ( $rand == 3 AND $values['ms_rps'] == 5) { $weThrew = 3; $youthrew = 5; $outcome = 111; }
			
			elseif ( $rand == 4 AND $values['ms_rps'] == 1) { $weThrew = 4; $youthrew = 1; $outcome = 111; }
			elseif ( $rand == 4 AND $values['ms_rps'] == 2) { $weThrew = 4; $youthrew = 2; $outcome = 222; }
			elseif ( $rand == 4 AND $values['ms_rps'] == 3) { $weThrew = 4; $youthrew = 3; $outcome = 111; }
			elseif ( $rand == 4 AND $values['ms_rps'] == 4) { $weThrew = 4; $youthrew = 4; $outcome = 333; }
			elseif ( $rand == 4 AND $values['ms_rps'] == 5) { $weThrew = 4; $youthrew = 5; $outcome = 222; }
			
			elseif ( $rand == 5 AND $values['ms_rps'] == 1) { $weThrew = 5; $youthrew = 1; $outcome = 222; }
			elseif ( $rand == 5 AND $values['ms_rps'] == 2) { $weThrew = 5; $youthrew = 2; $outcome = 111; }
			elseif ( $rand == 5 AND $values['ms_rps'] == 3) { $weThrew = 5; $youthrew = 3; $outcome = 222; }
			elseif ( $rand == 5 AND $values['ms_rps'] == 4) { $weThrew = 5; $youthrew = 4; $outcome = 111; }
			elseif ( $rand == 5 AND $values['ms_rps'] == 5) { $weThrew = 5; $youthrew = 5; $outcome = 333; }
			
			if ( $outcome == 111 )
			{
				if ( \IPS\Settings::i()->ms_sel_what_points == 0 )
		        {		
		            $member->shop_points = $member->shop_points + $getItem->price * 2;
		            $member->save();
		        }
                else
                {
                    $member->evox_points = $member->evox_points + $getItem->price * 2;
		            $member->save();
                }
			}
			elseif ( $outcome == 222 )
			{
			    
			}
			elseif ( $outcome == 333 )
			{
				if ( \IPS\Settings::i()->ms_sel_what_points == 0 )
		        {		
		            $member->shop_points = $member->shop_points + $getItem->price;
		            $member->save();
		        }
                else
                {
                    $member->evox_points = $member->evox_points + $getItem->price;
		            $member->save();
                }
			}
			
			$getItem->redeemed = 1;
		    $getItem->save();
			
			if ( $weThrew == 1 ){ $us = \IPS\Member::loggedIn()->language()->addToStack('ms_rock'); $ourIcon = '<i class="fa fa-hand-rock-o fa-5x"></i>';}
			elseif ( $weThrew == 2 ){ $us = \IPS\Member::loggedIn()->language()->addToStack('ms_paper'); $ourIcon = '<i class="fa fa-hand-stop-o fa-5x"></i>';}
			elseif ( $weThrew == 3 ){ $us = \IPS\Member::loggedIn()->language()->addToStack('ms_scissors'); $ourIcon = '<i class="fa fa-hand-scissors-o fa-5x"></i>';}
			elseif ( $weThrew == 4 ){ $us = \IPS\Member::loggedIn()->language()->addToStack('ms_lizard'); $ourIcon = '<i class="fa fa-hand-lizard-o fa-5x"></i>';}
			elseif ( $weThrew == 5 ){ $us = \IPS\Member::loggedIn()->language()->addToStack('ms_spock'); $ourIcon = '<i class="fa fa-hand-spock-o fa-5x"></i>';}
			
			if ( $youthrew == 1 ){ $you = \IPS\Member::loggedIn()->language()->addToStack('ms_rock'); $yourIcon = '<i class="fa fa-hand-rock-o fa-5x"></i>';}
			elseif ( $youthrew == 2 ){ $you = \IPS\Member::loggedIn()->language()->addToStack('ms_paper'); $yourIcon = '<i class="fa fa-hand-stop-o fa-5x"></i>';}
			elseif ( $youthrew == 3 ){ $you = \IPS\Member::loggedIn()->language()->addToStack('ms_scissors'); $yourIcon = '<i class="fa fa-hand-scissors-o fa-5x"></i>';}
			elseif ( $youthrew == 4 ){ $you = \IPS\Member::loggedIn()->language()->addToStack('ms_lizard'); $yourIcon = '<i class="fa fa-hand-lizard-o fa-5x"></i>';}
			elseif ( $youthrew == 5 ){ $you = \IPS\Member::loggedIn()->language()->addToStack('ms_spock'); $yourIcon = '<i class="fa fa-hand-spock-o fa-5x"></i>';}
			
			if ( $outcome == 111 ){ $result = \IPS\Member::loggedIn()->language()->addToStack('ms_win'); $amount = $getItem->price * 2; }
			elseif ( $outcome == 222 ){ $result = \IPS\Member::loggedIn()->language()->addToStack('ms_lose'); $amount = 0; }
			elseif ( $outcome == 333 ){ $result = \IPS\Member::loggedIn()->language()->addToStack('ms_draw'); $amount = $getItem->price; }
			
			$reward = new \IPS\membersshop\Rewards;
			$reward->item_id = $item_id;
			$reward->member_id = $member->member_id;
			$reward->time_date = time();
			$reward->reward = $amount;
			$reward->name = $getItem->name;
		    $reward->description = $getItem->description;
			$reward->save();
			
			\IPS\Output::i()->title = \IPS\Member::loggedIn()->language()->addToStack('ms_rps_result');
		    \IPS\Output::i()->breadcrumb[] = array( NULL, \IPS\Member::loggedIn()->language()->addToStack('ms_rps_result') );
			\IPS\Output::i()->output = \IPS\Theme::i()->getTemplate( 'shop', 'membersshop' )->rpsls( $us, $you, $result, $ourIcon, $yourIcon, $amount );
		
		}
		else
		{
			\IPS\Output::i()->output = $form->customTemplate( array( call_user_func_array( array( \IPS\Theme::i(), 'getTemplate' ), array( 'forms', 'core' ) ), 'popupTemplate' ) );
		}
	}
	
	protected function pinTopic()
	{
		$member = \IPS\Member::loggedIn();
		$id  = intval( \IPS\Request::i()->id );
		$td  = intval( \IPS\Request::i()->td );
		$tid = intval( \IPS\Request::i()->tid );
		try
		{
			$getItem = \IPS\membersshop\Purchases::load( $id );
			if ( $getItem->redeemed == 1 OR $getItem->member_id !== $member->member_id )
			{
				\IPS\Output::i()->error( 'ms_node_error', 'MEMBERSHOP/1', 403, '' );
			}
		}
		catch( \UnderflowException $ex )
		{
			\IPS\Output::i()->error( 'node_error', '2C163/8', 403, '' );
		}
		
		$topic = \IPS\forums\Topic::load( $tid );
		if ( $topic->pinned ) 
		{
			\IPS\Output::i()->error( 'ms_already_pinned', 'MEMBERSHOP/1', 403, '' );
		}
		
		$topic->pinned = 1;
		$topic->save();
		
		$pinned = new \IPS\membersshop\Pinned;
		$pinned->member_id = $member->member_id;
		$pinned->topic_id = $tid;
		$pinned->time_date = time();
		$pinned->expired = 0;
		$pinned->expire_time_date = \IPS\DateTime::create()->add( new \DateInterval( 'P' . $getItem->extra_field . 'D' ) )->getTimestamp();
		$pinned->save();
		
		$getItem->redeemed = 1;
		$getItem->save();
		
		$reward = new \IPS\membersshop\Rewards;
		$reward->item_id = 11;
		$reward->member_id = $member->member_id;
		$reward->time_date = time();
		$reward->name = $getItem->name;
		$reward->reward = $tid;
		$reward->description = $getItem->description;
		$reward->save();
		
		$expire = \IPS\DateTime::create()->setTimestamp( \IPS\DateTime::create()->add( new \DateInterval( 'P' . $getItem->extra_field . 'D' ) )->getTimestamp() );
		$topic = \IPS\forums\Topic::load( $tid );
		
		$outputLang = ( $expire ) ? \IPS\Member::loggedIn()->language()->addToStack('ms_pinned_topic_popup', FALSE, array( 'sprintf' => array( $topic->title, $expire ) ) ) : 'saved';
   	    \IPS\Output::i()->redirect( ( !empty( $_SERVER['HTTP_REFERER'] ) ) ? \IPS\Http\Url::createFromString( $_SERVER['HTTP_REFERER'] ) : \IPS\Http\Url::internal( '' ), $outputLang );
		
	}
	
	protected function featureFile()
	{
		if ( \IPS\Application::appIsEnabled( 'downloads') )
		{
			$member = \IPS\Member::loggedIn();
			$id  = intval( \IPS\Request::i()->id );
			$td  = intval( \IPS\Request::i()->td );
			$fid = intval( \IPS\Request::i()->fid );
			try
			{
				$getItem = \IPS\membersshop\Purchases::load( $id );
				if ( $getItem->redeemed == 1 OR $getItem->member_id !== $member->member_id )
				{
					\IPS\Output::i()->error( 'ms_node_error', 'MEMBERSHOP/1', 403, '' );
				}
			}
			catch( \UnderflowException $ex )
			{
				\IPS\Output::i()->error( 'node_error', '2C163/8', 403, '' );
			}
		
			$file = \IPS\downloads\File::load( $fid );
			
			if ( $file->featured ) 
			{
				\IPS\Output::i()->error( 'ms_already_featured', 'MEMBERSHOP/1', 403, '' );
			}
			
			$file->featured = 1;
			$file->save();
		
			$featured = new \IPS\membersshop\Featured;
			$featured->member_id = $member->member_id;
			$featured->fid = $file->id;
			$featured->time_date = time();
			$featured->expired = 0;
			$featured->expire_time_date = \IPS\DateTime::create()->add( new \DateInterval( 'P' . $getItem->extra_field . 'D' ) )->getTimestamp();
			$featured->save();
		
			$getItem->redeemed = 1;
			$getItem->save();
		
			$reward = new \IPS\membersshop\Rewards;
			$reward->item_id = 12;
			$reward->member_id = $member->member_id;
			$reward->time_date = time();
			$reward->name = $getItem->name;
			$reward->reward = $file->id;
		    $reward->description = $getItem->description;
			$reward->save();
			
			$expire = \IPS\DateTime::create()->setTimestamp( \IPS\DateTime::create()->add( new \DateInterval( 'P' . $getItem->extra_field . 'D' ) )->getTimestamp() );
			$getFile = \IPS\downloads\File::load( $fid );
			
			$outputLang = ( $expire ) ? \IPS\Member::loggedIn()->language()->addToStack('ms_featured_file_popup', FALSE, array( 'sprintf' => array( $getFile->name, $expire ) ) ) : 'saved';
   		    \IPS\Output::i()->redirect( ( !empty( $_SERVER['HTTP_REFERER'] ) ) ? \IPS\Http\Url::createFromString( $_SERVER['HTTP_REFERER'] ) : \IPS\Http\Url::internal( '' ), $outputLang );
		
		}
		else
		{
			\IPS\Output::i()->error( 'ms_downloads_error', 'MEMBERSHOP/1', 403, '' );
		}
	}
	
	protected function upgradeUsergroup()
	{
		$member = \IPS\Member::loggedIn();
		$id  = intval( \IPS\Request::i()->id );
		$item_id  = intval( \IPS\Request::i()->item_id );
		$timeDate = intval( \IPS\Request::i()->td );
		
		if ( $member->isAdmin() AND $member->modPermission() )
		{
			\IPS\Output::i()->error( 'ms_your_usergroup_is_protected', 'MEMBERSHOP/1', 403, '' );
		}
		
		try
		{
			$count = \IPS\membersshop\Upgrades::count();
            if ( $count > 0 )
			{
                \IPS\Output::i()->error( 'ms_wait_before_upgrading', 'MEMBERSHOP/1', 403, '' );
			}
		}
		catch( \UnderflowException $ex )
		{
			\IPS\Output::i()->error( 'node_error', '2C163/8', 403, '' );
		}
		
		try
		{
			$getItem = \IPS\membersshop\Purchases::load( $id );
			if ( $getItem->redeemed == 1 OR $getItem->member_id !== $member->member_id )
			{
				\IPS\Output::i()->error( 'ms_node_error', 'MEMBERSHOP/1', 403, '' );
			}
		}
		catch( \UnderflowException $ex )
		{
			\IPS\Output::i()->error( 'node_error', '2C163/8', 403, '' );
		}
		
		$group = \IPS\Member\Group::load( $getItem->minRand );
		
		$upgradeMembership = new \IPS\membersshop\Upgrades;
		$upgradeMembership->member_id = $member->member_id;
		$upgradeMembership->original_group = $member->member_group_id;
		$upgradeMembership->reward_group = $group->g_id;
		$upgradeMembership->expired = 0;
		$upgradeMembership->time_date = time();
		if ( $getItem->maxRand == 0 )
		{
		    $upgradeMembership->value = $getItem->minRand;
			$upgradeMembership->period = 'D';
			$amount = \IPS\DateTime::create()->add( new \DateInterval( 'P' . $getItem->extra_field . 'D' ) )->getTimestamp();
			$upgradeMembership->expire_time_date = $amount;
		}
		if ( $getItem->maxRand == 1 )
		{
		    $upgradeMembership->value = $getItem->minRand;
			$upgradeMembership->period = 'M';
			$amount = \IPS\DateTime::create()->add( new \DateInterval( 'P' . $getItem->extra_field . 'M' ) )->getTimestamp();
			$upgradeMembership->expire_time_date = $amount;
		}
		if ( $getItem->maxRand == 2 )
		{
		    $upgradeMembership->value = $getItem->minRand;
			$upgradeMembership->period = 'Y';
			$amount = \IPS\DateTime::create()->add( new \DateInterval( 'P' . $getItem->extra_field . 'Y' ) )->getTimestamp();
			$upgradeMembership->expire_time_date = $amount;
		}
		$upgradeMembership->save();
		
		$member->member_group_id = $group->g_id;
		$member->save();
		
		$getItem->redeemed = 1;
		$getItem->save();
		
		$reward = new \IPS\membersshop\Rewards;
		$reward->item_id = $item_id;
		$reward->member_id = $member->member_id;
		$reward->time_date = time();
		$reward->name = $getItem->name;
		$reward->description = $getItem->description;
		$reward->save();
		
		$items = \IPS\membersshop\Rewards::load( $reward->id );
		$notification = new \IPS\Notification( \IPS\Application::load( 'membersshop' ), 'rewards_reward', $items, array ( $items ) );
		$notification->recipients->attach( $member );
		$notification->send();
		
		\IPS\Output::i()->redirect( ( !empty( $_SERVER['HTTP_REFERER'] ) ) ? \IPS\Http\Url::createFromString( $_SERVER['HTTP_REFERER'] ) : \IPS\Http\Url::internal( '' ) );
	}
	
	protected function randomTrophy()
	{
		if ( \IPS\Application::appIsEnabled( 'trophies') )
		{
			$member = \IPS\Member::loggedIn();
			$id  = intval( \IPS\Request::i()->id );
			$item_id  = intval( \IPS\Request::i()->item_id );
			$timeDate = intval( \IPS\Request::i()->td );
			try
			{
				$getItem = \IPS\membersshop\Purchases::load( $id );
				if ( $getItem->redeemed == 1 OR $getItem->member_id !== $member->member_id )
				{
					\IPS\Output::i()->error( 'ms_node_error', 'MEMBERSHOP/1', 403, '' );
				}
			}
			catch( \UnderflowException $ex )
			{
				\IPS\Output::i()->error( 'node_error', '2C163/8', 403, '' );
			}
		
			$count = (int) \IPS\Db::i()->select( 'count(*)', 'trophies_trophy', array( 'id IS NOT NULL' ) )->first();
			foreach ( \IPS\Db::i()->select( '*', 'trophies_trophy', array( 'sort=? AND enabled=1', rand(1,$count) ), NULL, 1, NULL ) as $trophy )
			{
				try
			    {
				    $getTrophy = \IPS\Db::i()->select( 'id', 'trophies_members', array( 'member_id=? AND trophy_id=?', $member->member_id, $trophy['id'] ), NULL, NULL, NULL )->first();
				    $gotTrophy = TRUE;
				}
				catch( \Exception $ex )
				{
					$gotTrophy = FALSE;
				}
				if ( $gotTrophy )
				{
					$getItem->redeemed = 1;
					$getItem->save();
					\IPS\Output::i()->redirect( \IPS\Http\Url::internal( 'app=membersshop&module=shop&controller=myitems' ), 'ms_trophy_try_again' );
			    }
				else
				{
			        $reward = new \IPS\membersshop\Rewards;
					$reward->item_id = $item_id;
					$reward->member_id = $member->member_id;
					$reward->time_date = time();
					$reward->name = $getItem->name;
					$reward->description = $getItem->description;
					$reward->save();
			        
					\IPS\Db::i()->insert( 'trophies_members', array ( 'member_id' => $member->member_id, 'date' => time(), 'data' => '[]', 'trophy_id' => $trophy['id'], 'awarded_by' => 0, 'hidden' => 0, 'replaced_by_other_trophy' => 0 ) );
				
					$getItem->redeemed = 1;
					$getItem->save();
						
					$outputLang = ( $trophy['name_seo'] ) ? \IPS\Member::loggedIn()->language()->addToStack('ms_trophy_won', FALSE, array( 'sprintf' => array( $trophy['name_seo'] ) ) ) : 'saved';
   	                \IPS\Output::i()->redirect( ( !empty( $_SERVER['HTTP_REFERER'] ) ) ? \IPS\Http\Url::createFromString( $_SERVER['HTTP_REFERER'] ) : \IPS\Http\Url::internal( '' ), $outputLang );
				}
			}
		}
		else
		{
			\IPS\Output::i()->error( 'ms_trophies_error', 'MEMBERSHOP/1', 403, '' );
		}	
	}
	
	protected function customPurchase()
	{
		$member = \IPS\Member::loggedIn();
		$id  = intval( \IPS\Request::i()->id );
		$item_id  = intval( \IPS\Request::i()->item_id );
		$timeDate = intval( \IPS\Request::i()->td );
		try
		{
			$getItem = \IPS\membersshop\Purchases::load( $id );
			if ( $getItem->redeemed == 1 OR $getItem->member_id !== $member->member_id )
			{
				\IPS\Output::i()->error( 'ms_node_error', 'MEMBERSHOP/1', 403, '' );
			}
		}
		catch( \UnderflowException $ex )
		{
			\IPS\Output::i()->error( 'node_error', '2C163/8', 403, '' );
		}
				    
		$getItem->redeemed = 1;
		$getItem->save();
		
		$CustomPurchase = new \IPS\membersshop\Custom;
		$CustomPurchase->member_id = $member->member_id;
		$CustomPurchase->name = $getItem->name;
		$CustomPurchase->description = $getItem->description;
		$CustomPurchase->time_date = time();
		$CustomPurchase->item_id = $getItem->id;
		$CustomPurchase->price = $getItem->price;
		$CustomPurchase->icon = $getItem->icon;
		$CustomPurchase->save();
		
		if ( \IPS\Settings::i()->ms_cp_notifi_members_enable )
		{
			$items = \IPS\membersshop\Custom::load( $CustomPurchase->id );
			$members = \IPS\Member::load( \IPS\Settings::i()->ms_cp_notifi_members );
			$notification = new \IPS\Notification( \IPS\Application::load( 'membersshop' ), 'custom_purchase_mod', $items, array ( $items ) );
			$notification->recipients->attach( $members );
			$notification->send();
		}
		if ( \IPS\Settings::i()->ms_cp_email_members_enable )
		{
			try
			{
				$emailMessage = ( $member->name ) ? \IPS\Member::loggedIn()->language()->addToStack('ms_email_message', FALSE, array( 'sprintf' => array( $member->name, $getItem->name ) ) ) : 'saved';
   		    	$emailmember = \IPS\Member::load( \IPS\Settings::i()->ms_cp_email_members );
				$email = \IPS\Email::buildFromContent( \IPS\Member::loggedIn()->language()->addToStack('ms_email_subject'), $emailMessage, $emailMessage, \IPS\Email::TYPE_TRANSACTIONAL );
				$result = $email->_send( $emailmember->email, array(), array(), \IPS\Member::loggedIn()->email );
			}
			catch( \Exception $ex ){}
		}
		
		\IPS\Output::i()->redirect( ( !empty( $_SERVER['HTTP_REFERER'] ) ) ? \IPS\Http\Url::createFromString( $_SERVER['HTTP_REFERER'] ) : \IPS\Http\Url::internal( '' ), 'ms_custom_item_redeemed' );
	}
	
	protected function uploadAvatar()
	{
		$member = \IPS\Member::loggedIn();
		$id  = intval( \IPS\Request::i()->id );
		$item_id  = intval( \IPS\Request::i()->item_id );
		$timeDate = intval( \IPS\Request::i()->td );
		try
		{
			$getItem = \IPS\membersshop\Purchases::load( $id );
			if ( $getItem->redeemed == 1 OR $getItem->member_id !== $member->member_id )
			{
				\IPS\Output::i()->error( 'ms_node_error', 'MEMBERSHOP/1', 403, '' );
			}
		}
		catch( \UnderflowException $ex )
		{
			\IPS\Output::i()->error( 'node_error', '2C163/8', 403, '' );
		}
		
		$photoVars = explode( ':', $member->group['g_photo_max_vars'] );
		$form = new \IPS\Helpers\Form( 'membersshop', 'Save' );
		$form->add( new \IPS\Helpers\Form\Upload( 'member_photo_upload', NULL, FALSE, array( 'image' => array( 'maxWidth' => $photoVars[1], 'maxHeight' => $photoVars[2] ), 'storageExtension' => 'core_Profile', 'maxFileSize' => $photoVars[0] ? $photoVars[0] / 1024 : NULL ), function( $val ) {

			if ( $val instanceof \IPS\File )
			{
				$image = \IPS\Image::create( $val->contents() );
				if( $image->isAnimatedGif and !$this->member->group['g_upload_animated_photos'] )
				{
					throw new \DomainException('member_photo_upload_no_animated');
				}
			}
		}, NULL, NULL, 'member_photo_upload' ) );
		
		if( $values = $form->values() )
		{
			$member->pp_photo_type  = 'custom';
			$member->pp_main_photo  = (string) $values['member_photo_upload'];
			$thumbnail = $values['member_photo_upload']->thumbnail( 'core_Profile', \IPS\PHOTO_THUMBNAIL_SIZE, \IPS\PHOTO_THUMBNAIL_SIZE, TRUE );
			$member->pp_thumb_photo = (string) $thumbnail;
			$member->photo_last_update = time();
			$member->save();
		    
			$getItem->redeemed = 1;
		    $getItem->save();
			
			$reward = new \IPS\membersshop\Rewards;
			$reward->item_id = $item_id;
			$reward->member_id = $member->member_id;
			$reward->time_date = time();
			$reward->name = $getItem->name;
		    $reward->description = $getItem->description;
			$reward->save();
			
			\IPS\Output::i()->redirect( ( !empty( $_SERVER['HTTP_REFERER'] ) ) ? \IPS\Http\Url::createFromString( $_SERVER['HTTP_REFERER'] ) : \IPS\Http\Url::internal( '' ), 'ms_uploaded_avatar' );
		}
		else
		{
			\IPS\Output::i()->output = $form->customTemplate( array( call_user_func_array( array( \IPS\Theme::i(), 'getTemplate' ), array( 'forms', 'core' ) ), 'popupTemplate' ) );
		}
	}
	
	protected function goAnonymous()
	{
		$member = \IPS\Member::loggedIn();
		$id  = intval( \IPS\Request::i()->id );
		$item_id  = intval( \IPS\Request::i()->item_id );
		$timeDate = intval( \IPS\Request::i()->td );
		try
		{
			$getItem = \IPS\membersshop\Purchases::load( $id );
			if ( $getItem->redeemed == 1 OR $getItem->member_id !== $member->member_id )
			{
				\IPS\Output::i()->error( 'ms_node_error', 'MEMBERSHOP/1', 403, '' );
			}
		}
		catch( \UnderflowException $ex )
		{
			\IPS\Output::i()->error( 'node_error', '2C163/8', 403, '' );
		}
		
		$getItem->redeemed = 1;
		$getItem->save();
			
		\IPS\Session::i()->setAnon();
		
		\IPS\Output::i()->redirect( ( !empty( $_SERVER['HTTP_REFERER'] ) ) ? \IPS\Http\Url::createFromString( $_SERVER['HTTP_REFERER'] ) : \IPS\Http\Url::internal( '' ), 'ms_gone_anonymous' );
	}
	
	protected function guesstheNumber()
	{
		$member = \IPS\Member::loggedIn();
		$id  = intval( \IPS\Request::i()->id );
		$item_id  = intval( \IPS\Request::i()->item_id );
		$timeDate = intval( \IPS\Request::i()->td );
		try
		{
			$getItem = \IPS\membersshop\Purchases::load( $id );
			if ( $getItem->redeemed == 1 OR $getItem->member_id !== $member->member_id )
			{
				\IPS\Output::i()->error( 'ms_node_error', 'MEMBERSHOP/1', 403, '' );
			}
		}
		catch( \UnderflowException $ex )
		{
			\IPS\Output::i()->error( 'node_error', '2C163/8', 403, '' );
		}
		
		$form = new \IPS\Helpers\Form( 'membersshop', \IPS\Member::loggedIn()->language()->addToStack('ms_guess') );
		$form->add( new \IPS\Helpers\Form\Number('ms_guess_number', NULL, FALSE, array( 'min' => 1, 'max' => $getItem->maxRand ), NULL, NULL, NULL, 'ms_guess_number' ));
	
		if( $values = $form->values() )
		{
			$reward = new \IPS\membersshop\Rewards;
			
			$rand = rand( 1, $getItem->maxRand );
			if ( $values['ms_guess_number'] > $rand )
			{
				$text = \IPS\Member::loggedIn()->language()->addToStack('ms_too_high');
			}
			elseif ( $values['ms_guess_number'] < $rand )
			{
				$text = \IPS\Member::loggedIn()->language()->addToStack('ms_too_low');
			}
			elseif ( $values['ms_guess_number'] = $rand )
			{
				$text = \IPS\Member::loggedIn()->language()->addToStack('ms_will_done_you_won');
				$winnings = $getItem->price * $getItem->minRand;
				if ( \IPS\Settings::i()->ms_sel_what_points == 0 )
		        {
					$reward->reward = $winnings;
		            $member->shop_points = $member->shop_points + $winnings;
		            $member->save();
		        }
                else
                {
					$reward->reward = $winnings;
                    $member->evox_points = $member->evox_points + $winnings;
		            $member->save();
                }
			}
			$getItem->redeemed = 1;
			$getItem->save();
			
			$reward->item_id = $item_id;
			$reward->member_id = $member->member_id;
			$reward->time_date = time();
			$reward->name = $getItem->name;
		    $reward->description = $getItem->description;
			$reward->save();
			
			
			\IPS\Output::i()->redirect( ( !empty( $_SERVER['HTTP_REFERER'] ) ) ? \IPS\Http\Url::createFromString( $_SERVER['HTTP_REFERER'] ) : \IPS\Http\Url::internal( '' ), $text );
		}
		else
		{
			\IPS\Output::i()->output = $form->customTemplate( array( call_user_func_array( array( \IPS\Theme::i(), 'getTemplate' ), array( 'forms', 'core' ) ), 'popupTemplate' ) );
		}		
	}
}