<?php


namespace IPS\membersshop\modules\front\stats;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * main
 */
class _main extends \IPS\Dispatcher\Controller
{
	/**
	 * Execute
	 *
	 * @return	void
	 */
	public function execute()
	{
		parent::execute();
		if ( !\IPS\Request::i()->isAjax() AND !\IPS\Settings::i()->IPSDev_Copyright_Remover )
		{
		    $output = "<div style='clear:both;text-align:center;position:absolute;bottom:5px;width:95%;z-index:99999;'><a href='https://www.ipsdev.org/' target='_blank' class='ipsType_light ipsType_smaller'>Members Shop By IPSDev</a></div>";
		    \IPS\Output::i()->output .= $output;  
		}
	}

	/**
	 * ...
	 *
	 * @return	void
	 */
	 
	protected function manage()
	{
		$chart = new \IPS\Helpers\Chart\Database( \IPS\Http\Url::internal( "app=membersshop&module=stats&controller=main" ), 'membersshop_rewards', 'time_date', '', array( 
			'isStacked' => FALSE,
			'backgroundColor' 	=> '#ffffff',
			'colors'			=> array( '#10967e', '#ea7963', '#de6470' ),
			'hAxis'				=> array( 'gridlines' => array( 'color' => '#f5f5f5' ) ),
			'lineWidth'			=> 1,
			'areaOpacity'		=> 0.4
		), 
			'AreaChart',
			'daily' 
		);
		$chart->addSeries( \IPS\Member::loggedIn()->language()->addToStack('ms_stats_points_gained'), 'number', 'SUM(reward)' );
		$chart->title = \IPS\Member::loggedIn()->language()->addToStack( 'ms_shop_stats_points_period' );
		$chart->availableTypes = array( 'AreaChart', 'LineChart', 'ColumnChart', 'BarChart' );
		$chart->where = array( 'item_id=? or item_id=? or item_id=? or item_id=? or item_id=? or item_id=? or item_id=? or item_id=? or item_id=? or item_id=?', 5, 10, 16, 17, -1, -2, -3, -4, -5, -6  );
		if( \IPS\Request::i()->noheader OR \IPS\Request::i()->isAjax() )
		{
			\IPS\Output::i()->output = (string) $chart;
			return;
		}
		
		$limit = 10;
		$number = 1;
		
		if ( \IPS\Settings::i()->ms_sel_what_points == 0 )
		{
		    $mostPointsSort = 'core_members.shop_points DESC';
		}
		else
		{
			$mostPointsSort = 'core_members.evox_points DESC';
		}
		$mostPointsSelect = \IPS\Db::i()->select(
			'*',
			'core_members',
			'member_id',
			$mostPointsSort,
			$limit
		);

		$mostPoints = new \IPS\Patterns\ActiveRecordIterator(
			$mostPointsSelect,
			'\IPS\Member'
		);
		
		$mostPointsGained = array();
		foreach ( \IPS\Db::i()->select( array( 'member_id', 'reward' ), 'membersshop_rewards', array( 'item_id=? or item_id=? or item_id=? or item_id=? or item_id=? or item_id=?', -1, -2, -3, -4, -5, -6 ) ) as $rep )
		{
			if ( !isset( $mostPointsGained[ $rep['member_id'] ] ) )
			{
				$mostPointsGained[ $rep['member_id'] ] = $rep['reward'];
			}
			else
			{
				$mostPointsGained[ $rep['member_id'] ] += $rep['reward'];
			}
		}
		arsort( $mostPointsGained );
		$mostPointsGained = array_slice( $mostPointsGained, 0, $limit, TRUE );
		
		$mostRewards = array();
		foreach ( \IPS\Db::i()->select( array( 'member_id', 'reward' ), 'membersshop_rewards', array( 'item_id IS NOT NULL' ) ) as $rep )
		{
			if ( !isset( $mostRewards[ $rep['member_id'] ] ) )
			{
				$mostRewards[ $rep['member_id'] ] = $number;
			}
			else
			{
				$mostRewards[ $rep['member_id'] ] += $number;
			}
		}
		arsort( $mostRewards );
		$mostRewards = array_slice( $mostRewards, 0, $limit, TRUE );
		
		\IPS\Output::i()->sidebar['enabled'] = FALSE;
		\IPS\Output::i()->jsFiles = array_merge( \IPS\Output::i()->jsFiles, \IPS\Output::i()->js( 'admin.js', 'global' ) );
		\IPS\Output::i()->breadcrumb[] = array( NULL, \IPS\Member::loggedIn()->language()->addToStack( 'ms_main_stats', FALSE, array(  ) ) );
		\IPS\Output::i()->title = \IPS\Member::loggedIn()->language()->addToStack( 'ms_members_shop_stats' );
		\IPS\Output::i()->output = \IPS\Theme::i()->getTemplate( 'stats', 'membersshop' )->points( $chart, $limit, $mostPoints, $mostPointsGained, $mostRewards );
	}
	
	protected function shopitems()
	{
		$chart = new \IPS\Helpers\Chart\Database( \IPS\Http\Url::internal( "app=membersshop&module=stats&controller=main&do=shopitems" ), 'membersshop_purchases', 'time_date', '', array( 
			'isStacked' => FALSE,
			'backgroundColor' 	=> '#ffffff',
			'colors'			=> array( '#10967e', '#ea7963' ),
			'hAxis'				=> array( 'gridlines' => array( 'color' => '#f5f5f5' ) ),
			'lineWidth'			=> 1,
			'areaOpacity'		=> 0.4
		), 
			'AreaChart',
			'daily' 
		);
		$chart->addSeries( \IPS\Member::loggedIn()->language()->addToStack('ms_stats_spent'), 'number', 'SUM(price)' );
		$chart->addSeries( \IPS\Member::loggedIn()->language()->addToStack( 'ms_stats_purchases' ), 'number', 'COUNT(*)' );
		$chart->title = \IPS\Member::loggedIn()->language()->addToStack( 'ms_shop_stats_period' );
		$chart->availableTypes = array( 'AreaChart', 'LineChart', 'ColumnChart', 'BarChart' );
		
		if( \IPS\Request::i()->noheader OR \IPS\Request::i()->isAjax() )
		{
			\IPS\Output::i()->output = (string) $chart;
			return;
		}
		
		$limit = 10;
		
		$number = 1;
		$mostPurchases = array();
		foreach ( \IPS\Db::i()->select( array( 'member_id', 'id' ), 'membersshop_purchases', array( 'member_id>0', ) ) as $rep )
		{
			if ( !isset( $mostPurchases[ $rep['member_id'] ] ) )
			{
				$mostPurchases[ $rep['member_id'] ] = $number;
			}
			else
			{
				$mostPurchases[ $rep['member_id'] ] += $number;
			}
		}
		arsort( $mostPurchases );
		$mostPurchases = array_slice( $mostPurchases, 0, $limit, TRUE );
		
		$topContributors = array();
		foreach ( \IPS\Db::i()->select( array( 'member_id', 'price' ), 'membersshop_purchases', array( 'member_id>0', ) ) as $rep )
		{
			if ( !isset( $topContributors[ $rep['member_id'] ] ) )
			{
				$topContributors[ $rep['member_id'] ] = $rep['price'];
			}
			else
			{
				$topContributors[ $rep['member_id'] ] += $rep['price'];
			}
		}
		arsort( $topContributors );
		$topContributors = array_slice( $topContributors, 0, $limit, TRUE );
		

		$mostSent = array();
		foreach ( \IPS\Db::i()->select( array( 'sender_id', 'id' ), 'membersshop_sent_items', array( 'sender_id>0', ) ) as $rep )
		{
			if ( !isset( $mostSent[ $rep['sender_id'] ] ) )
			{
				$mostSent[ $rep['sender_id'] ] = $number;
			}
			else
			{
				$mostSent[ $rep['sender_id'] ] += $number;
			}
		}
		arsort( $mostSent );
		$mostSent = array_slice( $mostSent, 0, $limit, TRUE );
		
		\IPS\Output::i()->sidebar['enabled'] = FALSE;
		\IPS\Output::i()->jsFiles = array_merge( \IPS\Output::i()->jsFiles, \IPS\Output::i()->js( 'admin.js', 'global' ) );
		\IPS\Output::i()->breadcrumb[] = array( NULL, \IPS\Member::loggedIn()->language()->addToStack( 'ms_shop_stats', FALSE, array(  ) ) );
		\IPS\Output::i()->title = \IPS\Member::loggedIn()->language()->addToStack( 'ms_members_shop_stats' );
		\IPS\Output::i()->output = \IPS\Theme::i()->getTemplate( 'stats', 'membersshop' )->shop( $chart, $mostPurchases, $limit, $topContributors, $mostSent );
	}
	
	protected function rewards()
	{
		$chart = new \IPS\Helpers\Chart\Database( \IPS\Http\Url::internal( "app=membersshop&module=stats&controller=main&do=rewards" ), 'membersshop_rewards', 'time_date', '', array( 
			'isStacked' => FALSE,
			'backgroundColor' 	=> '#ffffff',
			'colors'			=> array( '#10967e', '#ea7963', '#de6470' ),
			'hAxis'				=> array( 'gridlines' => array( 'color' => '#f5f5f5' ) ),
			'lineWidth'			=> 1,
			'areaOpacity'		=> 0.4
		), 
			'AreaChart',
			'daily' 
		);
		$chart->addSeries( \IPS\Member::loggedIn()->language()->addToStack('ms_stats_points_won'), 'number', 'SUM(reward)' );
		$chart->addSeries( \IPS\Member::loggedIn()->language()->addToStack('ms_stats_boxes_opened'), 'number', 'count(*)' );
		$chart->title = \IPS\Member::loggedIn()->language()->addToStack( 'ms_shop_stats_points_period' );
		$chart->availableTypes = array( 'AreaChart', 'LineChart', 'ColumnChart', 'BarChart' );
		$chart->where = array( 'item_id=? or item_id=? or item_id=? or item_id=?', 5, 10, 16, 17);
		if( \IPS\Request::i()->noheader OR \IPS\Request::i()->isAjax() )
		{
			\IPS\Output::i()->output = (string) $chart;
			return;
		}
		
		$limit = 10;
		$number = 1;
		
		$topPointsWinner = array();
		foreach ( \IPS\Db::i()->select( array( 'member_id', 'reward' ), 'membersshop_rewards', array( 'item_id=5 or item_id=10 or item_id=16 or item_id=17', ) ) as $rep )
		{
			if ( !isset( $topPointsWinner[ $rep['member_id'] ] ) )
			{
				$topPointsWinner[ $rep['member_id'] ] = $rep['reward'];
			}
			else
			{
				$topPointsWinner[ $rep['member_id'] ] += $rep['reward'];
			}
		}
		arsort( $topPointsWinner );
		$topPointsWinner = array_slice( $topPointsWinner, 0, $limit, TRUE );
		
		
		$BroughtMostItemsForPoints = array();
		foreach ( \IPS\Db::i()->select( array( 'member_id', 'reward' ), 'membersshop_rewards', array( 'item_id=5 or item_id=10 or item_id=16 or item_id=17', ) ) as $rep )
		{
			if ( !isset( $BroughtMostItemsForPoints[ $rep['member_id'] ] ) )
			{
				$BroughtMostItemsForPoints[ $rep['member_id'] ] = $number;
			}
			else
			{
				$BroughtMostItemsForPoints[ $rep['member_id'] ] += $number;
			}
		}
		arsort( $BroughtMostItemsForPoints );
		$BroughtMostItemsForPoints = array_slice( $BroughtMostItemsForPoints, 0, $limit, TRUE );
		
		$LatestrewardedPoints = \IPS\Db::i()->select( '*', 'membersshop_rewards', array( 'item_id=5 or item_id=10 or item_id=16 or item_id=17', ), 'time_date DESC', $limit);
		
		\IPS\Output::i()->sidebar['enabled'] = FALSE;
		\IPS\Output::i()->jsFiles = array_merge( \IPS\Output::i()->jsFiles, \IPS\Output::i()->js( 'admin.js', 'global' ) );
		\IPS\Output::i()->breadcrumb[] = array( NULL, \IPS\Member::loggedIn()->language()->addToStack( 'ms_gamble_stats', FALSE, array(  ) ) );
		\IPS\Output::i()->title = \IPS\Member::loggedIn()->language()->addToStack( 'ms_members_shop_stats' );
		\IPS\Output::i()->output = \IPS\Theme::i()->getTemplate( 'stats', 'membersshop' )->rewards( $chart, $topPointsWinner, $limit, $BroughtMostItemsForPoints, $LatestrewardedPoints );
	}
}