<?php
/**
 * @brief		addMonthlyPoints Task
 * @author		<a href='https://www.invisioncommunity.com'>Invision Power Services, Inc.</a>
 * @copyright	(c) Invision Power Services, Inc.
 * @license		https://www.invisioncommunity.com/legal/standards/
 * @package		Invision Community
 * @subpackage	membersshop
 * @since		29 Nov 2017
 */

namespace IPS\membersshop\tasks;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * addMonthlyPoints Task
 */
class _addMonthlyPoints extends \IPS\Task
{
	/**
	 * Execute
	 *
	 * If ran successfully, should return anything worth logging. Only log something
	 * worth mentioning (don't log "task ran successfully"). Return NULL (actual NULL, not '' or 0) to not log (which will be most cases).
	 * If an error occurs which means the task could not finish running, throw an \IPS\Task\Exception - do not log an error as a normal log.
	 * Tasks should execute within the time of a normal HTTP request.
	 *
	 * @return	mixed	Message to log or NULL
	 * @throws	\IPS\Task\Exception
	 */
	public function execute()
	{
		$add = \IPS\Settings::i()->ms_monthly_points;
		$time = time();
		$oneMonthAgo = \IPS\DateTime::create()->sub( new \DateInterval( 'P1M' ) )->getTimestamp();
		
		if ( \IPS\Settings::i()->ms_activate_monthly ) 
		{
			if ( \IPS\Settings::i()->ms_sel_what_points == 0 )
			{
			    \IPS\Db::i()->update( 'core_members', "shop_points=shop_points+$add", array( 'last_visit BETWEEN ? AND ?', $oneMonthAgo, $time ) );
			}
			else
			{
			    \IPS\Db::i()->update( 'core_members', "evox_points=evox_points+$add", array( 'last_visit BETWEEN ? AND ?', $oneMonthAgo, $time ) );
			}
			return TRUE;
		}
		else
		{
			return 'Nothing Done';
		}
	}
	
	/**
	 * Cleanup
	 *
	 * If your task takes longer than 15 minutes to run, this method
	 * will be called before execute(). Use it to clean up anything which
	 * may not have been done
	 *
	 * @return	void
	 */
	public function cleanup()
	{
		
	}
}